// SNR
/*
 * Estimation of the signal-to-noise ratio function:
 *
 * SNR(f,g) = E(f^2) / E((f - g)^2)
 *
 * where f is a distribution function, g is a sample, and E(.) is the expected
 * value of the argument. The denominator is the mean square error, which we
 * approximate with the variance of the noise.
 *
 * Source PCL\src\modules\processes\ImageIntegration\ImageIntegrationInstance.cpp
 */

function SNREvaluation ( img )
{
   var E = new NoiseEvaluation( img );
   var snr = img.meanOfSquares(img.selectedRect)/ E.sigma / E.sigma;
   var db = 10 * Math.log10(snr);
   Console.writeln(format("SNR = %.3e, %.2f db ", snr, db));
}

function NoiseEvaluation( img )
{
   var a, n = 4, m = 0.01*img.selectedRect.area;
   for ( ;; )
   {
      a = img.noiseMRS( n );
      if ( a[1] >= m )
         break;
      if ( --n == 1 )
      {
         console.writeln( "<end><cbr>** Warning: No convergence in MRS noise evaluation routine - using k-sigma noise estimate." );
         a = img.noiseKSigma();
         break;
      }
   }
   this.sigma = a[0]; // estimated stddev of Gaussian noise
   this.count = a[1]; // number of pixels in the noise pixels set
   this.layers = n;   // number of layers used for noise evaluation
}

function main()
{
   // Get access to the current active image window.
   var window = ImageWindow.activeWindow;
   if ( window.isNull )
      throw new Error( "No active image" );

   console.show();
   console.writeln( "<end><cbr><br><b>" + window.currentView.fullId + "</b>" );
   console.writeln( "Calculating SNR" );
   console.flush();

   console.abortEnabled = true;

   var img = window.currentView.image;
   for ( var c = 0; c < img.numberOfChannels; ++c )
   {
      console.writeln( "<end><cbr><br>* Channel #", c );
      console.flush();
      img.selectedChannel = c;
      SNREvaluation ( img );
      console.flush();
   }
}

main();

